from PyQt5.QtWidgets import QAction, QMenu, QInputDialog, QLineEdit
from PyQt5.QtCore import QSettings

import PyQtAds as QtAds

CHILD_PREFIX = "Child-"

class DockInDockManager(QtAds.CDockManager):
    def __init__(self, parent: 'DockInDockWidget'):
        super().__init__()
        self.__parent = parent

    def parent(self) -> 'DockInDockWidget':
        return self.__parent
    
    def fillViewMenu(self, menu: QMenu, move_to: 'dict[DockInDockManager]') -> None:
        from dockindock import DockInDockWidget # Prevent cyclic import
        
        widgets_map = self.dockWidgetsMap()
        for key, value in widgets_map.items():
            widget = value.widget()
            action = value.toggleViewAction()
            
            if isinstance(widget, DockInDockWidget):
                sub_menu = menu.addMenu(key)
                
                sub_menu.addAction(action)
                sub_menu.addSeparator()
                
                widget.setupMenu(sub_menu, move_to)
            else:
                menu.addAction(action)
                
        if self.parent().canCreateNewGroups():
            # see how this works, to create it in the right place, 
            # and also to have load perspective work when some groups are missing
            menu.addSeparator()
            menu.addAction(CreateChildDockAction(self.__parent, menu))

            if self.parent().getTopLevelDockWidget().getManager() != self:
                menu.addAction(DestroyGroupAction( self.parent, menu))
                
    def fillMoveMenu(self, menu: QMenu, move_to: 'list[DockInDockManager]') -> None:
        widgets_map = self.dockWidgetsMap()
        for key, value in widgets_map.items():
            sub_menu = menu.addMenu(key)
            
            for mgr in move_to:
                # iterate over all possible target managers
                if mgr == self:
                    pass # if dock is already in mgr, no reason to move it there
                elif mgr == DockInDockManager.dockInAManager(value):
                    pass # if target is the group itself, can't move it there, would make no sense
                else:
                    sub_menu.addAction(MoveDockWidgetAction(value, mgr, sub_menu))
    
    def addPerspectiveRec(self, name: str) -> None:
        managers = self.allManagers(True, True)
        
        for child in managers:
            child.addPerspective(name)
            
    def openPerspectiveRec(self, name: str) -> None:
        managers = self.allManagers(True, True)
        
        for child in managers:
            child.openPerspective(name)
            
    def getGroupName(self) -> str:
        return self.parent().objectName()
        
    def getPersistGroupName(self) -> str:
        group = "Top"
        if self.getGroupName():
            group = CHILD_PREFIX + self.getGroupName()
        return group
        
    def getGroupNameFromPersistGroupName(self, persist_group_name) -> str:
        if persist_group_name.startswith(CHILD_PREFIX):
            persist_group_name = persist_group_name[len(CHILD_PREFIX):]
        else:
            assert False
        return persist_group_name
        
    def loadPerspectivesRec(self, settings: QSettings) -> None:
        children = self.allManagers(True, True)
        
        for mgr in children:
            settings.beginGroup(mgr.getPersistGroupName())
            mgr.loadPerspectives(settings)
            settings.endGroup()
            
    def savePerspectivesRec(self, settings: QSettings) -> None:
        children = self.allManagers(True, True)
        
        for mgr in children:
            settings.beginGroup(mgr.getPersistGroupName())
            mgr.savePerspectives(settings)
            settings.endGroup()
            
    def removePerspectivesRec(self, settings: QSettings) -> None:
        children = self.allManagers(True, True)
        
        for mgr in children:
            child.removePerspectives(child.perspectiveNames())
            
    @staticmethod
    def dockInAManager(widget) -> 'DockInDockManager':
        from dockindock import DockInDockWidget # Prevent cyclic import
        
        dock_widget =  widget.widget() if widget else None
        return dock_widget.getManager() if isinstance(dock_widget, DockInDockWidget) else None
        
    def childManagers(self, managers: 'list[DockInDockManager]', rec: bool) -> None:
        widgets = self.getWidgetsInGUIOrder()
        for widget in widgets:
            as_mgr = DockInDockManager.dockInAManager(widget)
            if as_mgr:
                managers.append(as_mgr)
                if rec:
                    as_mgr.childManagers(managers, rec)
                    
    def allManagers(self, include_self: bool, rec: bool) -> 'list[DockInDockManager]':
        managers = []
        if include_self:
            managers.append(self)
        self.childManagers(managers, rec)
        return managers
        
    def allDockWidgets(self, include_self: bool, rec: bool) -> 'list[tuple[DockInDockManager, QtAds.CDockWidget]]':
        widgets = []
        for mgr in self.allManagers(include_self, rec):
            for widget in mgr.getWidgetsInGUIOrder():
                widgets.append((mgr, widget))
        return widgets
        
    def getGroupContents(self) -> 'dict[str, list[str]]':
        result = {}
        managers = self.allManagers(True, True)
        for mgr in managers:
            result[mgr.getPersistGroupName()] = mgr.dockWidgetsMap().keys()
        return result
        
    def getInsertDefaultPos(self) -> QtAds.CDockAreaWidget:
        default_pos = None
        if self.dockAreaCount() != 0:
            default_pos = self.dockArea(self.dockAreaCount()-1)
        return default_pos
        
    def getWidgetsInGUIOrder(self) -> 'list[QtAds.CDockWidget]':
        result = []
        for i in range(self.dockAreaCount()):
            for widget in self.dockArea(i).dockWidgets():
                result.append(widget)
        return result
        
    
class CreateChildDockAction(QAction):
    def __init__(self, dock_in_dock: 'DockInDockWidget', menu: QMenu):
        super().__init__("New group...", menu)
        self.__dock_in_dock = dock_in_dock
        self.triggered.connect(self.createGroup)
        
    def createGroup(self) -> None:
        name = ""
        while True:
            name, ok = QInputDialog.getText(None, self.text(), "Enter group name", QLineEdit.Normal, name)
            if ok:
                error = ""
                if self.__dock_in_dock.getTopLevelDockWidget():
                    error = self.__dock_in_dock.getTopLevelDockWidget().getGroupNameError(name)
                else:
                    assert False
                    
                if not error:
                    self.__dock_in_dock.createGroup(name, None)
                    break
                else:
                    QMessageBox.critical(None, "Error", error)
                    continue
            else:
                break
        
class DestroyGroupAction(QAction):
    def __init__(self, widget: 'DockInDockWidget', menu: QMenu):
        super().__init__("Destroy" + widget.getManager().getGroupName(), menu)
        self.__widget = widget
        self.triggered.connect(self.destroyGroup)
        
    def destroyGroup(self) -> None:
        self.__widget.getTopLevelDockWidget().destroyGroup(self.__widget)
        
        
class MoveDockWidgetAction(QAction):
    def __init__(self, widget: 'DockInDockWidget', move_to: DockInDockManager, menu: QMenu):
        super().__init__(menu)
        self.__widget = widget
        self.__move_to = move_to
        
        if move_to.parent().isTopLevel():
            self.setText("To top")
        else:
            self.setText(f"To {move_to.parent().objectName()}")
        self.triggered.connect(self._move)
        
    def _move(self) -> None:
        self.move(self.__widget, self.__move_to)
        
    def move(self, widget: QtAds.CDockWidget, move_to: QtAds.CDockManager) -> None:
        if widget and move_to:
            widget.dockManager().removeDockWidget(widget)
            move_to.addDockWidget(QtAds.CenterDockWidgetArea, widget, move_to.getInsertDefaultPos())
        else:
            assert False



from PyQt6.QtWidgets import QMenu
from PyQt6.QtGui import QAction



class LoadPerspectiveAction(QAction):
    def __init__(self, parent: QMenu, name: str, dock_manager: "DockInDockWidget"):
        super().__init__(name, parent)
        self.name = name
        self.dock_manager = dock_manager

        self.triggered.connect(self.load)

    def load(self):
        self.dock_manager.getPerspectivesManager().openPerspective(
            self.name, self.dock_manager
        )


class RemovePerspectiveAction(QAction):
    def __init__(self, parent: QMenu, name: str, dock_manager: "DockInDockWidget"):
        super().__init__(name, parent)
        self.name = name
        self.dock_manager = dock_manager

        self.triggered.connect(self.remove)

    def remove(self):
        self.dock_manager.getPerspectivesManager().removePerspective(self.name)



from pathlib import Path
import sys
import atexit


from PyQt6.QtWidgets import QApplication, QMainWindow, QLabel
from PyQt6.QtCore import Qt

sys.path.append(str(Path(__file__).parent))
from perspectives import PerspectivesManager
from dockindock import DockInDockWidget


class MainWindow(QMainWindow):
    def __init__(self, parent=None):
        super().__init__(parent)

        self.perspectives_manager = PerspectivesManager("persist")
        self.resize(400, 400)
        self.dock_manager = DockInDockWidget(
            self, self.perspectives_manager, can_create_new_groups=True
        )
        self.setCentralWidget(self.dock_manager)
        self.dock_manager.attachViewMenu(self.menuBar().addMenu("View"))

        previous_dock_widget = None
        for i in range(3):
            l = QLabel()
            l.setWordWrap(True)
            l.setAlignment(Qt.AlignmentFlag.AlignTop | Qt.AlignmentFlag.AlignLeft)
            l.setText("Lorem ipsum dolor sit amet, consectetuer adipiscing elit. ")

            previous_dock_widget = self.dock_manager.addTabWidget(
                l, f"Top label {i}", previous_dock_widget
            )

        last_top_level_dock = previous_dock_widget

        for j in range(2):
            group_manager, _ = self.dock_manager.createGroup(
                f"Group {j}", last_top_level_dock
            )

            previous_dock_widget = None

            for i in range(3):
                # Create example content label - this can be any application specific widget
                l = QLabel()
                l.setWordWrap(True)
                l.setAlignment(Qt.AlignmentFlag.AlignTop | Qt.AlignmentFlag.AlignLeft)
                l.setText("Lorem ipsum dolor sit amet, consectetuer adipiscing elit. ")

                previous_dock_widget = group_manager.addTabWidget(
                    l, f"ZInner {j}/{i}", previous_dock_widget
                )

            # create sub-group
            sub_group, _ = group_manager.createGroup(
                f"SubGroup {j}", previous_dock_widget
            )
            previous_dock_widget = None
            for i in range(3):
                # Create example content label - this can be any application specific widget
                l = QLabel()
                l.setWordWrap(True)
                l.setAlignment(Qt.AlignmentFlag.AlignTop | Qt.AlignmentFlag.AlignLeft)
                l.setText("Lorem ipsum dolor sit amet, consectetuer adipiscing elit. ")

                previous_dock_widget = sub_group.addTabWidget(
                    l, f"SubInner {j}/{i}", previous_dock_widget
                )

        self.perspectives_manager.loadPerspectives()
        atexit.register(self.cleanup)

    def cleanup(self):
        self.perspectives_manager.savePerspectives()


if __name__ == "__main__":
    app = QApplication(sys.argv)

    w = MainWindow()
    w.show()
    app.exec()



import os
import sys

from PyQt6 import uic
from PyQt6.QtCore import Qt
from PyQt6.QtWidgets import QApplication, QLabel

import PyQt6Ads as QtAds

UI_FILE = os.path.join(os.path.dirname(__file__), "MainWindow.ui")
MainWindowUI, MainWindowBase = uic.loadUiType(UI_FILE)


class MainWindow(MainWindowUI, MainWindowBase):
    def __init__(self, parent=None):
        super().__init__(parent)

        self.setupUi(self)

        # Create the dock manager. Because the parent parameter is a QMainWindow
        # the dock manager registers itself as the central widget.
        self.dock_manager = QtAds.CDockManager(self)

        # Create example content label - this can be any application specific
        # widget
        l = QLabel()
        l.setWordWrap(True)
        l.setAlignment(Qt.AlignmentFlag.AlignTop | Qt.AlignmentFlag.AlignLeft)
        l.setText("Lorem ipsum dolor sit amet, consectetuer adipiscing elit. ")

        # Create a dock widget with the title Label 1 and set the created label
        # as the dock widget content
        dock_widget = QtAds.CDockWidget("Label 1")
        dock_widget.setWidget(l)

        # Add the toggleViewAction of the dock widget to the menu to give
        # the user the possibility to show the dock widget if it has been closed
        self.menuView.addAction(dock_widget.toggleViewAction())

        # Add the dock widget to the top dock widget area
        self.dock_manager.addDockWidget(QtAds.DockWidgetArea.TopDockWidgetArea, dock_widget)


if __name__ == "__main__":
    app = QApplication(sys.argv)

    w = MainWindow()
    w.show()
    app.exec()



"""Delocate wheel file."""

import re
import sys
from subprocess import run
from pathlib import Path
import shutil


def main() -> None:
    if sys.platform != "darwin":
        return

    dest_dir, wheel, *_ = sys.argv[1:]

    # unzip the wheel to a tmp directory
    tmp_dir = Path(wheel).parent / "tmp"
    shutil.unpack_archive(wheel, tmp_dir, format="zip")

    # fix the rpath in the tmp directory
    if sys.platform == "darwin":
        for so in Path(tmp_dir).rglob("*.so"):
            fix_rpath(so)

    # re-zip the tmp directory and place it at dest_dir / wheel.name
    new_wheel = Path(dest_dir) / Path(wheel).name
    shutil.make_archive(new_wheel, "zip", tmp_dir)
    # remove the .zip extension
    shutil.move(f"{new_wheel}.zip", new_wheel)
    assert new_wheel.exists()
    print("Placed the repaired wheel at", new_wheel)


RPATH_RE = re.compile(r"^\s*path (.+) \(offset \d+\)$", re.MULTILINE)


def fix_rpath(so: Path, new_rpath: str = "@loader_path/PyQt6/Qt6/lib") -> None:
    # delete all current rpaths
    current_rpath = run(["otool", "-l", str(so)], capture_output=True, text=True)
    for rpath in RPATH_RE.findall(current_rpath.stdout):
        run(["install_name_tool", "-delete_rpath", rpath, so], check=True)

    # add new rpath
    run(["install_name_tool", "-add_rpath", new_rpath, so], check=True)
    print(f"Updated RPATH for {so} to {new_rpath}")


if __name__ == "__main__":
    main()



import os
import sys

from PyQt5 import uic
from PyQt5.QtCore import Qt, QTimer, QDir, QSignalBlocker
from PyQt5.QtGui import QCloseEvent, QIcon
from PyQt5.QtWidgets import (QApplication, QLabel, QCalendarWidget, QFrame, QTreeView,
                             QTableWidget, QFileSystemModel, QPlainTextEdit, QToolBar,
                             QWidgetAction, QComboBox, QAction, QSizePolicy, QInputDialog)

import PyQtAds as QtAds

UI_FILE = os.path.join(os.path.dirname(__file__), 'mainwindow.ui')
MainWindowUI, MainWindowBase = uic.loadUiType(UI_FILE)
    
class MainWindow(MainWindowUI, MainWindowBase):

    def __init__(self, parent=None):
        super().__init__(parent)

        self.setupUi(self)
 
        QtAds.CDockManager.setConfigFlag(QtAds.CDockManager.OpaqueSplitterResize, True)
        QtAds.CDockManager.setConfigFlag(QtAds.CDockManager.XmlCompressionEnabled, False)
        QtAds.CDockManager.setConfigFlag(QtAds.CDockManager.FocusHighlighting, True)
        self.dock_manager = QtAds.CDockManager(self)
        
        # Set central widget
        text_edit = QPlainTextEdit()
        text_edit.setPlaceholderText("This is the central editor. Enter your text here.")
        central_dock_widget = QtAds.CDockWidget("CentralWidget")
        central_dock_widget.setWidget(text_edit)
        central_dock_area = self.dock_manager.setCentralWidget(central_dock_widget)
        central_dock_area.setAllowedAreas(QtAds.DockWidgetArea.OuterDockAreas)
        
        # create other dock widgets
        table = QTableWidget()
        table.setColumnCount(3)
        table.setRowCount(10)
        table_dock_widget = QtAds.CDockWidget("Table 1")
        table_dock_widget.setWidget(table)
        table_dock_widget.setMinimumSizeHintMode(QtAds.CDockWidget.MinimumSizeHintFromDockWidget)
        table_dock_widget.resize(250, 150)
        table_dock_widget.setMinimumSize(200, 150)
        table_area = self.dock_manager.addDockWidget(QtAds.DockWidgetArea.LeftDockWidgetArea, table_dock_widget)
        self.menuView.addAction(table_dock_widget.toggleViewAction())
        
        table = QTableWidget()
        table.setColumnCount(5)
        table.setRowCount(1020)
        table_dock_widget = QtAds.CDockWidget("Table 2")
        table_dock_widget.setWidget(table)
        table_dock_widget.setMinimumSizeHintMode(QtAds.CDockWidget.MinimumSizeHintFromDockWidget)
        table_dock_widget.resize(250, 150)
        table_dock_widget.setMinimumSize(200, 150)
        table_area = self.dock_manager.addDockWidget(QtAds.DockWidgetArea.BottomDockWidgetArea, table_dock_widget, table_area)
        self.menuView.addAction(table_dock_widget.toggleViewAction())

        properties_table = QTableWidget()
        properties_table.setColumnCount(3)
        properties_table.setRowCount(10)
        properties_dock_widget = QtAds.CDockWidget("Properties")
        properties_dock_widget.setWidget(properties_table)
        properties_dock_widget.setMinimumSizeHintMode(QtAds.CDockWidget.MinimumSizeHintFromDockWidget)
        properties_dock_widget.resize(250, 150)
        properties_dock_widget.setMinimumSize(200, 150)
        self.dock_manager.addDockWidget(QtAds.DockWidgetArea.RightDockWidgetArea, properties_dock_widget, central_dock_area)
        self.menuView.addAction(properties_dock_widget.toggleViewAction())
        
        self.create_perspective_ui()
        
    def create_perspective_ui(self):
        save_perspective_action = QAction("Create Perspective", self)
        save_perspective_action.triggered.connect(self.save_perspective)
        perspective_list_action = QWidgetAction(self)
        self.perspective_combobox = QComboBox(self)
        self.perspective_combobox.setSizeAdjustPolicy(QComboBox.AdjustToContents)
        self.perspective_combobox.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        self.perspective_combobox.activated[str].connect(self.dock_manager.openPerspective)
        perspective_list_action.setDefaultWidget(self.perspective_combobox)
        self.toolBar.addSeparator()
        self.toolBar.addAction(perspective_list_action)
        self.toolBar.addAction(save_perspective_action)
        
    def save_perspective(self):
        perspective_name, ok = QInputDialog.getText(self, "Save Perspective", "Enter Unique name:")
        if not ok or not perspective_name:
            return
        
        self.dock_manager.addPerspective(perspective_name)
        blocker = QSignalBlocker(self.perspective_combobox)
        self.perspective_combobox.clear()
        self.perspective_combobox.addItems(self.dock_manager.perspectiveNames())
        self.perspective_combobox.setCurrentText(perspective_name)
        
    def closeEvent(self, event: QCloseEvent):
        self.dock_manager.deleteLater()
        super().closeEvent(event)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    
    w = MainWindow()
    w.show()
    app.exec_()
